#! /usr/bin/env python
###############################################################################
#
#  Script name                  : MMInfo.py
#
#  Dependencies                 : ComFuncs.py
#                                 ComDefs.py
#                                 MMComunications.py
#                    
#  Objective                    : Script used to login to the management port (6090)
#                                 of the MM and run dot commands to get information
#                                 about the MM and the BladeCenter chassis
#                    
#  Written by                   : Mike Nolterieke
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/01/04 - Initial revision.
#                                 RKC - 07/26/04 - use getopt instead of optparse
#                     
###############################################################################
"""
   Script used to login to the management port (6090) of the MM and run dot commands
   to get information about the MM and the BladeCenter chassis
"""
try:
    import sys, getopt, string
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.MMCommunications import MMCom
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# The last error that occurred
lastError   = ComDefs.SUCCESS

# The MMCom class
mmCom       = ""
logger      = ""
isTest = ComDefs.FALSE
cleanExit = ComDefs.TRUE

class mmBayInfo:
    installed = ComDefs.FALSE
    active = ComDefs.FALSE
    mainbuild = ""
    remotebuild = ""
    mainstatus = ComDefs.FALSE
    remotestatus = ComDefs.FALSE
    committed = ComDefs.FALSE
    restarted = ComDefs.FALSE

class statusTable:
    mm = []
    # mm[0] should never be used so we can index by bay number
    for i in xrange(3):
        mm.append(mmBayInfo())

class MMStatus:

    table = statusTable()

    def __init__(self):
        """
        Constructer used to initiallize the class members
        """

    def gather(self):
        """
        Method used to gather the MMInfo and populate the class members
        """
        self.setInstalledMMBays()
        self.setActiveMMBay()
        self.setBuildIDs()
        if lastError == ComDefs.SUCCESS:
            if self.table.mm[self.returnActiveBay()].mainbuild >= ComDefs.REDUNDANT_MM_MAIN_CODELEVEL:
                #self.setRedundantMMBay()
                self.setRedundantBuildIDs()


    def setRestarted(self, bay):
        """
        """
        self.table.mm[bay].restarted = ComDefs.TRUE


    def setCommitted(self, bay):
        """
        """
        self.table.mm[bay].committed = ComDefs.TRUE


    def setMainStatus(self, bay):
        """
        """
        self.table.mm[bay].mainstatus = ComDefs.TRUE


    def setRemoteStatus(self, bay):
        """
        """
        self.table.mm[bay].remotestatus = ComDefs.TRUE


    def setActiveMMBay(self):
        """
        """
        activeBay = getActiveMMBay()
        #if lastError != ComDefs.SUCCESS:
        #    logger.log(ComDefs.GetLastError(lastError), ComDefs.TRUE)

        for i in xrange(2):
            j = i + 1
            if activeBay & j:
                self.table.mm[j].active = ComDefs.TRUE
            else:
                self.table.mm[j].active = ComDefs.FALSE


    def setRedundantMMBay(self):
        """
        """
        redMMbay = IsRedundantMMInstalled()
        #if lastError != ComDefs.SUCCESS:
        #    logger.log(ComDefs.GetLastError(lastError), ComDefs.TRUE)

        if redMMbay > 0:
            if redMMbay != self.returnInactiveBay():
                logger.log("We have conflicting results on which Management Module Bay is redundant")
            if self.table.mm[redMMbay].active != ComDefs.FALSE:
                logger.log("We have conflicting results on which Management Module Bay is active")
            if self.table.mm[redMMbay].installed != ComDefs.TRUE:
                logger.log("We have conflicting results on which Management Module Bays are installed")


    def setInstalledMMBays(self):
        """
        """
        installedBays = getInstalledMMBays()
        #if lastError != ComDefs.SUCCESS:
        #    logger.log(ComDefs.GetLastError(lastError), ComDefs.TRUE)

        for i in xrange(2):
            j = i + 1
            if installedBays & j:
                self.table.mm[j].installed = ComDefs.TRUE


    def setBuildIDs(self):
        for i in xrange(2):
            j = i + 1
            if self.table.mm[j].active == ComDefs.TRUE:
                self.table.mm[j].mainbuild = getBuildIDs(1)
                self.table.mm[j].remotebuild = getBuildIDs(3)


    def setRedundantBuildIDs(self):
        for i in xrange(2):
            j = i + 1
            if self.table.mm[j].active == ComDefs.FALSE and \
               self.table.mm[j].installed == ComDefs.TRUE:
                self.table.mm[j].mainbuild = getRedundantBuildIDs(1)
                self.table.mm[j].remotebuild = getRedundantBuildIDs(3)


    def getStatusTable(self):
        return self.table


    def returnInactiveBay(self):
        # Which bay is active
        for i in xrange(2):
            j = i + 1
            if self.table.mm[j].active == ComDefs.FALSE:
                break
        return j

        
    def returnActiveBay(self):
        # Which bay is active
        for i in xrange(2):
            j = i + 1
            if self.table.mm[j].active == ComDefs.TRUE:
                break
        return j
        

    def toString(self):
        """
        Displays the contents of the class memebers
        """

        print
        for i in xrange(2):
            j = i + 1
            print "Management Module Bay %d:" % j
            print "\tInstalled: %d" % self.table.mm[j].installed
            print "\tActive: %d" % self.table.mm[j].active
            print "\tMain Application Build: %s" % self.table.mm[j].mainbuild
            print "\tRemote Control Build: %s" % self.table.mm[j].remotebuild
            print "\tMain Application Updated: %s" % self.table.mm[j].mainstatus
            print "\tRemote Control Updated: %s" % self.table.mm[j].remotestatus
            print "\tRestarted: %d" % self.table.mm[j].restarted
            print "\tCommitted: %d" % self.table.mm[j].committed
            print

    
def connectToMM(params):
    """ Connect to the MM management port (6090) and login
    """
    # Use global variable
    global mmCom
    return (mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword))


def getInstalledMMBays():
    global mmCom, lastError

    installedMMbays = 0
    
    lastError = mmCom.sendCmd('R', '\x04\x05\x09', '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        installedMMbays = ord(response[29])
        
    return installedMMbays
    


def getActiveMMBay():
    global mmCom, lastError

    activeMMbay = 0
    
    lastError = mmCom.sendCmd('R', '\x04\x05\x09', '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        activeMMbay = ord(response[33])
        
    return activeMMbay


def getChassisModelType():
    global mmCom, lastError
    
    mtm = ""
    lastError = mmCom.sendCmd('R', '\x0D\x01\x01', '\x04\xA8\x28\x00\x0E\x00\x07\x00')
    
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()

        i = 0
        while i < len(response):
            if i < 4:
                mtm += "%s" % chr(ord(response[i]))
            else:
                pass
            i += 1
        return mtm
        
    # return error code if dot command fails
    return lastError

def getChassisHardwareRev():
    """ Get the BladeCenter chassis hardware revision
    Returns:
        A 1 digit build ID (1, 2, 3, etc...) of the chassis hardware
        revision or -1 if there was an error
    If 0 is returned, the error code will be stored in the lastError global variable
    """
    global mmCom, lastError
    hardwareRev = -1
    # Just for my own sanity...
    # 4.3.5.5 = VPD dot command
    # Data for the command is 6 bytes:
    #   Module Type         - 1 byte
    #   Slot Number         - 1 byte
    #   Read Address LSB    - 1 byte
    #   Read Address MSB    - 1 byte
    #   Read Length LSB     - 1 byte
    #   Read Length MSB     - 1 byte
    # Module Type   = /x06  - Chassis
    # Slot Number   = /x01  - Always slot 1 for the chassis
    # Read Address  = /x0065- \x00 is MSB and \x65 is LSB
    #   These bytes must be swapped when sending the command
    #   So, \x0065 becomes \x65\x00 in LSB, MSB format
    # Read Length   = \x0002- \x00 is MSB and \x02 is LSB
    #   These bytes must also be swapped when sending the command
    #   So, \x0001 becomes \x02\x00 in LSB, MSB format
    #   Note that we only need 1 byte of information but we are getting two back
    #   This is because the first byte is ALWAYS the VPD format (IBM, OEM, etc...)
    #   We can skip that byte and go right to the data
    # Send the command and check for an error
#    lastError = mmCom.sendCmd('R', '\x04\x03\x05\x05', '\x06\x01\x65\x00\x02\x00')
    lastError = mmCom.sendCmd('R', '\x0D\x01\x01', '\x04\xA8\x28\x00\x65\x00\x02\x00')
    
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()
        # The first byte [0] is the VPD format, we can skip that and go right
        # to the data [1]
        
        hardwareRev = int(ord(response[0]))
    # Return the hardware revision
    return hardwareRev

def IsRedundantMMInstalled():
    """ This command only works if the MM code level is > 52 """
    global logger, mmCom, lastError
    
    lastError = mmCom.sendCmd('R', '\x16\x02\x01', '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        rMMBay = ord(response[0])
        if rMMBay > 0:
            logger.log("Management Module in bay %s is redundant." % rMMBay)
        return rMMBay
    else:
        logger.log("Error %s returned from command." % lastError)
    return ComDefs.SUCCESS

def IsMMDhcp():
    """ Returns a boolean value dependent upon whether the MM
        is configured via DHCP or not.
    """
    global logger, mmCom, lastError, params
    
    lastError = mmCom.sendCmd('R', '\x04\x09\x01\x01\x10\x02', '')

    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        
        dhcpIPaddr = "%s.%s.%s.%s" % (ord(response[0]), ord(response[1]), ord(response[2]), ord(response[3]))

        if dhcpIPaddr != "0.0.0.0":
            return ComDefs.TRUE
        else:
            return ComDefs.FALSE
    
def getBuildIDs(whichOne=1):
    """ Get the build ID's of the MM Main Application, Boot ROM, or Remote Graphics
    Values of whichOne should be 1, 2, or 3 where:
        1 = Main Application
        2 = Boot ROM
        3 = Remote Control
    Returns:
        The 7 character build ID (in text) of the specified firmware or an empty
        string if there was an error
    If an empty string is returned, the error code will be stored in the lastError
    global variable
    """
    global mmCom, lastError
    
    buildID = ""
    # Make sure they gave us a valid argument
    if ((whichOne < 1) | (whichOne > 3)):
        lastError = ComDefs.MM_INVALID_CODE_LEVEL
        return buildID
    # Convert the argument to a hex string
    whichID = ComFuncs.intToHex(whichOne, ComDefs.TRUE)

    # Send the command
    lastError = mmCom.sendCmd('R', '\x04\x03\x05\x06' + whichID, '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        i = 0
        while i < len(response):
            if i < 7:
                buildID += "%s" % chr(ord(response[i]))
            else:
                pass
            i += 1
    try:
        tmp = int(buildID[4:-1])
    except ValueError:
        logger.log("Could not determine primary Management Module firmware build level,\nassuming 27A.")
        if whichOne == 1:
            buildID = ComDefs.MINIMUM_MM_MAIN_CODELEVEL
        elif whichOne == 2:
            buildID = ComDefs.MINIMUM_MM_BOOT_CODELEVEL
        elif whichOne == 3:
            buildID = ComDefs.MINIMUM_MM_REMOTE_CODELEVEL
        lastError = ComDefs.GENERAL_UPDATE_ERROR
    return buildID

def getNumSupportedIOModules():
    """ Get number of installed IO modules in chassis. """
    
    global mmCom, lastError
    supportedModules = 0
    
    lastError = mmCom.sendCmd('R', '\x04\x05\x09', '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        supportedModules = ord(response[19])
        
    return supportedModules
    
    
    
def getRedundantBuildIDs(whichOne=1):
    """ Get the build ID's of the MM Main Application, Boot ROM, or Remote Graphics
    Values of whichOne should be 1, 2, or 3 where:
        1 = Main Application
        2 = Boot ROM
        3 = Remote Control
    Returns:
        The 7 character build ID (in text) of the specified firmware or an empty
        string if there was an error
    If an empty string is returned, the error code will be stored in the lastError
    global variable
    """
    global mmCom, lastError
    
    buildID = ""
    # Make sure they gave us a valid argument
    if ((whichOne < 1) | (whichOne > 3)):
        lastError = ComDefs.MM_INVALID_CODE_LEVEL
        return buildID
    # Convert the argument to a hex string
    whichID = ComFuncs.intToHex(whichOne, ComDefs.TRUE)

    # Send the command
    # 22.2.4.3.5.6.x
    lastError = mmCom.sendCmd('R', '\x16\x02\x04\x03\x05\x06' + whichID, '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        i = 0
        while i < len(response):
            if i < 7:
                buildID += "%s" % chr(ord(response[i]))
            else:
                pass
            i += 1
    try:
        tmp = int(buildID[4:-1])
    except ValueError:
        logger.log("Could not determine redundant Management Module firmware build level,\nassuming 27A.")
        if whichOne == 1:
            buildID = ComDefs.MINIMUM_MM_MAIN_CODELEVEL
        elif whichOne == 2:
            buildID = ComDefs.MINIMUM_MM_BOOT_CODELEVEL
        elif whichOne == 3:
            buildID = ComDefs.MINIMUM_MM_REMOTE_CODELEVEL
        lastError = ComDefs.GENERAL_UPDATE_ERROR
    return buildID

def usage():
    '''
        Display the Usage Information to the screen
    '''

    print "Usage: MMInfo.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the Management"
    print "                           Module"
    print "  -u, --username=USERNAME  An administrator user name used to log into the"
    print "                           Management Module"
    print "  -p, --password=PASSWORD  The password of the administrator user name used to"
    print "                           log into the Management Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the MMInfo version number and exit"


def parse_args(opts, params):
    """
       Parse the arguments and options and handle them accordingly
    """

    global isTest, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        # Hidden test parameter, specify -t to use our known ip address, user name,
        # and passwords to test this script
        elif o in ("-t", "--test"):
            isTest = ComDefs.TRUE
        elif o in ("-i", "--ipaddress"):
            params.mmIPAddress = a
        elif o in ("-u", "--username"):
            params.mmUserName = a
        elif o in ("-p", "--password"):
            params.mmPassword = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS


def verify_args(params):
    """
       Verify that the required arguments have been supplied.
    """

    global cleanExit

    if params.mmIPAddress == "":
        print "The hostname or dotted IP address of the Management Module was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS


def main():
    ''' Main function used to run this script in standalone mode'''
    # Define using a global variable
    global mmCom, lastError, logger
    
    # Instanciate the class
    params = ComDefs.parameters()

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?htvVi:u:p:",
             ["?", "help", "test", "verbose", "version", "ipaddress=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(2)

    lastError = parse_args(opts, params)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # If we are in test mode (-t parameter), use our defaults and ignore the rest
    # of the parameters
    if isTest:
        # Populate the class
        params.mmIPAddress     = ""
        params.mmUserName   = ""
        params.mmPassword   = ""
        
    else:
        lastError = verify_args(params)
        if lastError != ComDefs.SUCCESS:
            return lastError

    logger = Logger(params) 
    # See if we are in debug mode
    mmCom = MMCom(params.debugLevel)
    mmCom.logger = logger
    
    # Get the MM info using the 6090 management port
    # Connect to the MM port 6090
    logger.log("Retrieve management module information.")
    logger.log("Opening connection and logging into the MM")
    if connectToMM(params) != ComDefs.SUCCESS:
        logger.log("There was an error connecting or logging into the MM!", ComDefs.TRUE)
        sys.exit(1)
    else:
        logger.log("Successfully connected and logged into the MM!")

    # Get the midplane hardware revision
    logger.log("Getting chassis hardware revision...")
    hwRev = getChassisHardwareRev()
    if hwRev == "":
        logger.log("Error getting chassis hardware revision!", ComDefs.TRUE)
    else:
        logger.log("Chassis hardware revision is: %s" % hwRev, ComDefs.TRUE)
    # JRH - Check to see if MM is configured with DHCP
    isDhcp = IsMMDhcp()
    if isDhcp == ComDefs.TRUE:
        logger.log("Management Module is configured via DHCP: True", ComDefs.TRUE)
    else:
        logger.log("Management Module is configured via DHCP: False", ComDefs.TRUE)

    logger.log("Getting the Primary Management Module build ID's...", ComDefs.TRUE)
    # Get the main application build ID
    buildID = getBuildIDs()
    if buildID == "":
        logger.log("Error getting Main Application build ID!", ComDefs.TRUE)
    else:
        logger.log("\tMain Application build ID is:\t%s" % buildID, ComDefs.TRUE)
    # Get the boot ROM build ID
    #buildID = getBuildIDs(2)
    #if buildID == "":
    #    logger.log("Error getting Boot ROM build ID!", ComDefs.TRUE)
    #else:
    #    logger.log("\tBoot ROM build ID is:\t\t%s" % buildID, ComDefs.TRUE)
    # Get the remote graphics build ID
    buildID = getBuildIDs(3)
    if buildID == "":
        logger.log("Error getting Remote Control build ID!", ComDefs.TRUE)
    else:
        logger.log("\tRemote Control build ID is:\t%s" % buildID, ComDefs.TRUE)
    
    # JRH - check to see if we have a redundant MM installed.
    if int(getBuildIDs(1)[4:-1]) > 51:
        rMMBay = IsRedundantMMInstalled()
        if rMMBay > 0:
            # Print redudant MM build id
            logger.log("Getting the Redundant Management Module build IDs...", ComDefs.TRUE)
            # Get the main application build ID
            buildID = getRedundantBuildIDs()
            if buildID == "":
                logger.log("Error getting Main Application build ID!", ComDefs.TRUE)
            else:
                logger.log("\tMain Application build ID is:\t%s" % buildID, ComDefs.TRUE)
            # Get the boot ROM build ID
            #buildID = getRedundantBuildIDs(2)
            #if buildID == "":
            #    logger.log("Error getting Boot ROM build ID!", ComDefs.TRUE)
            #else:
            #    logger.log("\tBoot ROM build ID is:\t\t%s" % buildID, ComDefs.TRUE)
            # Get the remote graphics build ID
            buildID = getRedundantBuildIDs(3)
            if buildID == "":
                logger.log("Error getting Remote Control build ID!", ComDefs.TRUE)
            else:
                logger.log("\tRemote Control build ID is:\t%s" % buildID, ComDefs.TRUE)
        else:
            logger.log("Redundant Management Module: Not Installed", ComDefs.TRUE)
    else:
        logger.log("Redundant Management Module: Unsupported", ComDefs.TRUE)

    if params.debugLevel > 2:
        mmtable = MMStatus()
        mmtable.gather()
        mmtable.toString()

    # Close the connection to the MM
    mmCom.close()
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)

